{% extends "base.html" %}
{% load i18n %}

{% block head %}
<script type="text/javascript" src="{{ STATIC_URL }}kobo/js/log_watcher.js"></script>
<script type="text/javascript" language="javascript">
document.log_watcher = new LogWatcher('{{ json_url }}', {{ offset }}, {{ task_finished }}, {{ next_poll }});
document.log_watcher.watch();
</script>
{% endblock %}

{% block content %}
<h2>{% trans 'Task' %} #{{ task.id }} - {{ log_name }}</h2>
<a href="{% url 'task/detail' task.id %}">{% trans "back to task" %} #{{ task.id }}</a><br />
<a href="?format=raw">{% trans 'download' %}</a>
<pre class="log" id="log">
{{ content }}
</pre>

{% endblock %}
