{% extends "base.html" %}
{% load i18n %}

{% block content %}
<h2>{% trans "Arch list" %}</h2>

<br />
{% include "pagination.html" %}
{% include "arch/list_include.html" %}
{% include "pagination.html" %}

{% endblock %}
